'use strict';

// This module handles the OpenLayers-based map

/* global appCore,MapModals */
function MapFind() {
    var self = this;

    var closed = false;

    self.open = function()
    {
        appCore.setActiveOverlay('find');

        $.get('find/find-tpl.html', function(data) {

            // Add template to DOM
            $('#app-container').append(data);

            // Translate it
            IMatchTranslate.translateElement('#find-container');

            // The Google/GN radio buttons.
            // Set from the current appCore.appConfig
            $('#form-find-location input[type=radio][name=find-provider]').val([appCore.IMatchConfig.searchProvider]);

            // Focus and select the find text
            $('#find-location').focus();
            $('#find-location').select();

            // We monitor changes in the find location and call the de-bouncer function.
            $('#find-location').on('input',function() { 
                delayedFind();
            });

            // If the user switches the search provider type, we reset the result list
            // and restart the search when there is a search term.
            $('input[type=radio][name=find-provider]').click(function() {
                appCore.IMatchConfig.searchProvider = this.value;
                
                // Update the corresponding setting in IMatch
                IMatch.setIMatchSetting('geo',appCore.IMSETTINGS.epGeoSearchService,this.value);

                $('#find-result').text('');

                var text = $('#find-location').val();
                if (text !== undefined)  {
                    delayedFind();
                }
            });

            // Bind the locate button for the target marker
            $('#form-find-target').submit(function(e) {
                e.preventDefault();
                positionTargetMarker();
            });

            $('#find-container-close').click(self.close);

            $('#find-location').val(appCore.appConfig.findText);
            $('#find-target-marker').val(appCore.appConfig.findTarget);
            
            $(document).keydown(function(e) {
                if (event.which === 27) { // ESC
                    e.preventDefault();
                    self.close();
                }
            });        

            // If we have a search term but the result is empty, 
            // we trigger the search        
            if ((appCore.appConfig.findText !== undefined) && (appCore.appConfig.findText.length > 0)) {
                var li = $('#find-result li');
                if ((li === undefined) || (li.length == 0)) {
                    delayedFind();
                }
            }

            $('#find-container').css('width','100%');
        });
    }

    self.close = function()
    {
        closed = true;

        $('#find-container').css('width','0%');
        $('#find-container').remove();
        
        appCore.setActiveOverlay('');
    }

    // Create a de-bouncer function. We link this to the change event for
    // the find input. As long as the method is called (the user typing),
    // the wrapped function is not called. If the user stops making changes
    // for 1 second, we call findAddress.
    var delayedFind = IMatch.debounce(function() {
        if (!closed) {
            findAddress();
        }
    },1000);


    // Lookup the best matches for the search string entered by the user.
    // Display the results in the list in the form.
    // This method is either called directly or via the de-bouncer
    function findAddress()
    {
        // Clear the results
        $('#find-result').text('');

        // Get the find text
        var text = $('#find-location').val();
        appCore.appConfig.findText = text;
        appCore.saveConfig();

        if (text.length) {

            if (appCore.IMatchConfig.searchProvider == 1) {
                var params = {
                    q : text,
                    maxRows : 5,
                    style: 'MEDIUM',
                    orderby: 'relevance',
                    lang: appCore.IMatchConfig.locale,
                    username : appCore.IMatchConfig.geoNamesKey
                }
                $.ajax({
                    method: 'GET',
                    dataType: "jsonp",
                    url: 'http://api.geonames.org/searchJSON',
                    data: params
                }).then(function(response) {
                    var ul = $('#find-result');
                    response.geonames.forEach(function(g) {

                        var name = g.name;
                        if ((g.adminName1 !== undefined) && g.adminName1.length) {
                            name += ', ' + g.adminName1;
                        }
                        if ((g.countryName !== undefined) && g.countryName.length) {
                            name += ', ' + g.countryName;
                        }
                        ul.append('<li><i class="fa fa-map-marker imatch-margin-r-10"></i><a href="#" data-lat="' + g.lat + '" data-lng="' + g.lng + '">' + name + '</a></li>');
                    }, this);

                    // Bind the as and center the map to the coordinates
                    $('#find-result a').click(function() {
                        var a = $(this);
                        var lat = a.attr('data-lat');
                        var lng = a.attr('data-lng');
                        appCore.getMapModule().positionTargetMarker(lng,lat);
                    });

                    //console.log(JSON.stringify(response,null,2));
                });
            }
            else {
                // Search Google
                var params = {
                    address : text,
                    language: appCore.IMatchConfig.locale
                }
                $.ajax({
                    method: 'GET',
                    url: 'https://maps.googleapis.com/maps/api/geocode/json',
                    data: params
                }).then(function(response) {
                    var ul = $('#find-result');
                    response.results.forEach(function(r) {
                        ul.append('<li><i class="fa fa-map-marker imatch-margin-r-10"></i><a href="#" data-lat="' + r.geometry.location.lat + '" data-lng="' + r.geometry.location.lng + '">' + r.formatted_address + '</li>');
                    }, this);

                    // Bind the as and center the map to the coordinates
                    $('#find-result a').click(function() {
                        var a = $(this);
                        var lat = a.attr('data-lat');
                        var lng = a.attr('data-lng');
                        appCore.getMapModule().positionTargetMarker(lng,lat);
                    });

                    //console.log(JSON.stringify(response,null,2));
                });
            }
        }

    }

   
    // This function is called when the user has entered a target marker
    // coordinate and clicked on the button to locate it on the map.
    function positionTargetMarker()
    {
        var text = $('#find-target-marker').val();
        appCore.appConfig.findTarget = text;
        appCore.saveConfig();
        
        if (text.length) {
            var coord = text.split(',');
            if (coord.length == 2) {
                appCore.getMapModule().positionTargetMarker(coord[1],coord[0]);
            }
        }
    }
}
